// model_meshes.fx
//
//// global.fx

#ifdef HAS_OPENGL_ES
#	ifdef VSHADER
		precision highp float;
		precision highp int;
#	endif
#	ifdef FSHADER
		precision mediump float;
		precision mediump int;
#	endif
#else
#	define highp
#	define lowp 
#	define mediump
#endif //HAS_OPENGL_ES

//

//--------------------------------------------------------------------------------------------------
// Uniforms
//--------------------------------------------------------------------------------------------------
uniform highp mat4		u_mModelViewProj;
uniform highp mat4		u_mModelView;
uniform highp mat3		u_mNormalView;

uniform mediump vec3	u_vLightViewDir;
uniform lowp vec3		u_vSceneDiffuse;
uniform lowp vec4		u_vSceneAmbient;

uniform mediump float	u_fTextCoord0TransX;
uniform lowp sampler2D	u_Texture0;

//--------------------------------------------------------------------------------------------------
// Varying
//--------------------------------------------------------------------------------------------------
varying lowp vec4		v_vColor;
varying mediump vec2	v_vTexCoord0;

//// fog.fx
//

#ifdef FOG

//--------------------------------------------------------------------------------------------------
// Uniforms
//--------------------------------------------------------------------------------------------------
uniform mediump float	u_fFogEnd;
uniform mediump float	u_fFogDensity;			// end-start for linear
uniform lowp vec3		u_vFogColor;

//--------------------------------------------------------------------------------------------------
// Varying
//--------------------------------------------------------------------------------------------------
varying lowp float		v_fFogShade;

#ifdef VSHADER
//==================================================================================================
//==================================================================================================
void MakeFogShade( mediump float fEyeDistance )
{
	v_fFogShade = clamp( (u_fFogEnd - fEyeDistance) / u_fFogDensity, 0.0, 1.0 );
}
#endif
#ifdef FSHADER
//==================================================================================================
//==================================================================================================
lowp vec4 FogShade( lowp vec4 vColor )
{
    return vec4( mix( u_vFogColor, vec3( vColor ), v_fFogShade ), vColor.a );
}
#endif

#endif	//FOG

//// packed_float.fx

const highp vec4	c_vPackFloatRGBAFactors		= vec4( 1.0, 255.0, 65025.0, 160581375.0 );
const highp vec4	c_vUnpackFloatRGBAFActors	= vec4( 1.0, 1.0/255.0, 1.0/65025.0, 1.0/160581375.0 );
//==================================================================================================
//==================================================================================================
highp vec4 PackFloatRGBA( float v )
{
  return fract( c_vPackFloatRGBAFactors * v );
}
//==================================================================================================
//==================================================================================================
float UnpackFloatRGBA( highp vec4 rgba ) 
{
  return dot( rgba, c_vUnpackFloatRGBAFActors );
}

//// shadowmap.fx

#ifdef SHADOWMAP

#ifdef SHADOWMAP_SAMPLERS
#	define ShadowSampler2D		sampler2DShadow
#	ifdef HAS_OPENGL_ES
#		extension GL_EXT_shadow_samplers : require
#	endif
#else
#	define ShadowSampler2D		sampler2D
#endif
//--------------------------------------------------------------------------------------------------
// Uniforms
//--------------------------------------------------------------------------------------------------
uniform highp mat4				u_mShadowProj[SHADOWMAP_NUM_SPLITS];
uniform highp ShadowSampler2D	u_sShadowTexture[SHADOWMAP_NUM_SPLITS];
uniform mediump vec3			u_vShadowViewSplits;					// The distance for each level in view space
//--------------------------------------------------------------------------------------------------
// Varying
//--------------------------------------------------------------------------------------------------
varying highp vec4				v_vPosView;

#ifdef VSHADER
//==================================================================================================
//==================================================================================================
void MakeShadowMapViewPos( mediump vec4 vPosView )
{
	v_vPosView		= vPosView;
}
#endif
#ifdef FSHADER
//==================================================================================================
//==================================================================================================
lowp float ShadowSample( ShadowSampler2D sShadowMap, highp vec4 vPosShadow )
{
#ifdef SHADOWMAP_SAMPLERS
	//----------------------------------------------------------------------------------------------
	//  When the hardware supports sampler2DShadow, the value returned from the sampler is a full
	// coverage value.  The sampling uses the z value of the texture coords to do the compare
	// after the sampling.  Also the hardware will do a 4x4 PCF if the texture filtering is set to
	// LINEAR.
	//----------------------------------------------------------------------------------------------
#	ifdef HAS_OPENGL_ES
		return shadow2DProjEXT( sShadowMap, vPosShadow );
#	else
		return shadow2DProj( sShadowMap, vPosShadow ).x;
#	endif
#else
	//----------------------------------------------------------------------------------------------
	//  The value retuned from a regular sampler2D is the z value for the light.  If the texture is 
	// a RGBA, we have to unpack into a float first.  Finally this z value is compare with the 
	// light z to give us a shade or not shaded answer.
	//----------------------------------------------------------------------------------------------
#ifdef SHADOWMAP_PACKED_FLOATRGBA
	highp float	fLightDepth	= UnpackFloatRGBA( texture2D( sShadowMap, vPosShadow.xy ) );
#else
	highp float	fLightDepth	= texture2D( sShadowMap, vPosShadow.xy ).z;
#endif

	return (vPosShadow.z <= fLightDepth) ? 1.0 : 0.0;
#endif	//SHADOWMAP_SAMPLERS
}
//==================================================================================================
//==================================================================================================
lowp vec4 ShadowMapMixAmbient( lowp vec4 vColor, lowp float fShade )
{
	return vColor * min( fShade + SHADOWMAP_AMBIENT, 1.0 );
}
//==================================================================================================
//==================================================================================================
lowp vec4 ShadowMapMixAmbientFade( lowp vec4 vColor, lowp float fShade )
{
	// Ramp the ambient for the shadows from SHADOWMAP_AMBIENT to 1.0 to make shadows fade.
	lowp float	fFade			= clamp( (length( v_vPosView ) - SHADOWMAP_FADE_START) / SHADOWMAP_FADE_DISTANCE, 0.0, 1.0 );
	lowp float	fShadowAmbient	= SHADOWMAP_AMBIENT + (SHADOWMAP_ONE_MIN_AMBIENT * fFade);

	return vColor * min( fShade + fShadowAmbient, 1.0 );
}
//==================================================================================================
//==================================================================================================
lowp vec4 ShadowMapShade( lowp vec4 vColor, highp vec4 vPosView )
{
	//----------------------------------------------------------------------------------------------
	//  Use the depth coord to determine which ShadowMap level we will use.  Each level has a 
	// projection and a texture.
	//----------------------------------------------------------------------------------------------
#if SHADOWMAP_NUM_SPLITS == 1

	lowp float	fShade = ShadowSample( u_sShadowTexture[0], u_mShadowProj[0] * vPosView );

	return ShadowMapMixAmbientFade( vColor, fShade );

#elif SHADOWMAP_NUM_SPLITS == 2

	if ( gl_FragCoord.z < u_vShadowViewSplits.x )
	{
		lowp float	fShade0 = ShadowSample( u_sShadowTexture[0], u_mShadowProj[0] * vPosView );

		return ShadowMapMixAmbient( vColor, fShade0 );
	}
	else
	{
		lowp float	fShade1 = ShadowSample( u_sShadowTexture[1], u_mShadowProj[1] * vPosView );

		return ShadowMapMixAmbientFade( vColor, fShade1 );
	}
#elif SHADOWMAP_NUM_SPLITS >= 3 

	if ( gl_FragCoord.z < u_vShadowViewSplits.x )
	{
		lowp float	fShade0 = ShadowSample( u_sShadowTexture[0], u_mShadowProj[0] * vPosView );

		return ShadowMapMixAmbient( vColor, fShade0 );
	}
	else
	if ( gl_FragCoord.z < u_vShadowViewSplits.y )
	{
		lowp float	fShade1 = ShadowSample( u_sShadowTexture[1], u_mShadowProj[1] * vPosView );

		return ShadowMapMixAmbient( vColor, fShade1 );
	}

	lowp float	fShade2 = ShadowSample( u_sShadowTexture[2], u_mShadowProj[2] * vPosView );

	return ShadowMapMixAmbientFade( vColor, fShade2 );
#endif
}
#endif	//FSHADER
#endif	//SHADOWMAP

#ifdef VSHADER
//--------------------------------------------------------------------------------------------------
// Vertex attributes
//--------------------------------------------------------------------------------------------------
attribute highp vec4	a_vPosition;
attribute mediump vec3	a_vNormal;
attribute mediump vec2	a_vTexCoord0;
//==================================================================================================
//==================================================================================================
void main()
{
	// Texture coordinate translation.
	v_vTexCoord0	= vec2( a_vTexCoord0.s + u_fTextCoord0TransX, a_vTexCoord0.t );

	// Directional lighting.
	lowp vec3	vNormal			= u_mNormalView * a_vNormal;
	lowp float	fDiffuseShade	= max( dot( vNormal, u_vLightViewDir ), 0.0 );
	lowp vec3	vDiffuse		= u_vSceneDiffuse *  fDiffuseShade;

	v_vColor	= u_vSceneAmbient + vec4( vDiffuse, 0.0 );

	mediump vec4	vPosView	= u_mModelView * a_vPosition;

#ifdef SHADOWMAP
	MakeShadowMapViewPos( vPosView );
#endif
#ifdef FOG
	MakeFogShade( abs( vPosView.z ) );
#endif

	gl_Position	= u_mModelViewProj * a_vPosition;
}
#endif
#ifdef FSHADER
//==================================================================================================
//==================================================================================================
void main()
{
	// Shade texture using per vertex lighting texture.
	lowp vec4	vColor	= v_vColor * texture2D( u_Texture0, v_vTexCoord0 );

#ifdef SHADOWMAP
	vColor	= ShadowMapShade( vColor, v_vPosView );
#endif
#ifdef FOG
	vColor	= FogShade( vColor );
#endif

	gl_FragColor	= vColor;
}
#endif
