// model_tress.fx
//
//// global.fx

#ifdef HAS_OPENGL_ES
#	ifdef VSHADER
		precision highp float;
		precision highp int;
#	endif
#	ifdef FSHADER
		precision mediump float;
		precision mediump int;
#	endif
#else
#	define highp
#	define lowp 
#	define mediump
#endif //HAS_OPENGL_ES

//

//--------------------------------------------------------------------------------------------------
// Uniforms
//--------------------------------------------------------------------------------------------------
uniform highp mat4		u_mModelViewProj;
uniform highp mat4		u_mModelView;
uniform highp mat3		u_mNormalView;

uniform mediump vec3	u_vLightViewDir;
uniform lowp vec3		u_vSceneDiffuse;
uniform lowp vec4		u_vSceneAmbient;

uniform lowp float		u_fAlphaRef;

uniform lowp sampler2D	u_Texture0;

//--------------------------------------------------------------------------------------------------
// Varying
//--------------------------------------------------------------------------------------------------
varying lowp vec4		v_vColor;
varying mediump vec2	v_vTexCoord0;

//// fog.fx
//

#ifdef FOG

//--------------------------------------------------------------------------------------------------
// Uniforms
//--------------------------------------------------------------------------------------------------
uniform mediump float	u_fFogEnd;
uniform mediump float	u_fFogDensity;			// end-start for linear
uniform lowp vec3		u_vFogColor;

//--------------------------------------------------------------------------------------------------
// Varying
//--------------------------------------------------------------------------------------------------
varying lowp float		v_fFogShade;

#ifdef VSHADER
//==================================================================================================
//==================================================================================================
void MakeFogShade( mediump float fEyeDistance )
{
	v_fFogShade = clamp( (u_fFogEnd - fEyeDistance) / u_fFogDensity, 0.0, 1.0 );
}
#endif
#ifdef FSHADER
//==================================================================================================
//==================================================================================================
lowp vec4 FogShade( lowp vec4 vColor )
{
    return vec4( mix( u_vFogColor, vec3( vColor ), v_fFogShade ), vColor.a );
}
#endif

#endif	//FOG


#ifdef VSHADER
//--------------------------------------------------------------------------------------------------
// Vertex attributes
//--------------------------------------------------------------------------------------------------
attribute highp vec4	a_vPosition;
attribute mediump vec3	a_vNormal;
attribute mediump vec2	a_vTexCoord0;
//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------
void main()
{
	v_vTexCoord0	= a_vTexCoord0;

	// Directional lighting.
	lowp vec3	vNormal			= u_mNormalView * a_vNormal;
	lowp float	fDiffuseShade	= max( dot( vNormal, u_vLightViewDir ), 0.0 );
	lowp vec3	vDiffuse		= u_vSceneDiffuse *  fDiffuseShade;

	v_vColor	= u_vSceneAmbient + vec4( vDiffuse, 0.0 );

#ifdef FOG
	mediump vec4	vPosView	= u_mModelView * a_vPosition;

	MakeFogShade( abs( vPosView.z ) );
#endif

	gl_Position	= u_mModelViewProj * a_vPosition;
}
#endif
#ifdef FSHADER
//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------
void main()
{
	vec4	vColor = v_vColor * texture2D( u_Texture0, v_vTexCoord0 );
    if ( vColor.a < u_fAlphaRef )
		discard;

#ifdef FOG
	vColor	= FogShade( vColor );
#endif
	gl_FragColor	= vColor;
}
#endif

