// terrain_splat_brushes2.fx
//
//// global.fx

#ifdef HAS_OPENGL_ES
#	ifdef VSHADER
		precision highp float;
		precision highp int;
#	endif
#	ifdef FSHADER
		precision mediump float;
		precision mediump int;
#	endif
#else
#	define highp
#	define lowp 
#	define mediump
#endif //HAS_OPENGL_ES

//

//--------------------------------------------------------------------------------------------------
// Uniforms
//--------------------------------------------------------------------------------------------------
uniform highp mat4		u_mModelViewProj;
uniform highp mat4		u_mModelView;

uniform lowp sampler2D	u_AlphaTexture;		// r = b0 / g = b1 / b = b2 / a = b3
uniform lowp sampler2D	u_BrushTexture[4];
uniform highp float		u_fBrushScale[4];
//--------------------------------------------------------------------------------------------------
// Varying
//--------------------------------------------------------------------------------------------------
varying highp vec2		v_vAlphaTexCoord;
varying highp vec2		v_vBrushTexCoord[4];
varying lowp float		v_fFade;

#ifdef VSHADER
//--------------------------------------------------------------------------------------------------
// Vertex attributes
//--------------------------------------------------------------------------------------------------
attribute highp vec4	a_vPosition;
attribute highp vec2	a_vTexCoord0;
//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------
void main()
{
	v_vAlphaTexCoord	= a_vTexCoord0;

	v_vBrushTexCoord[0]	= a_vTexCoord0 * u_fBrushScale[0];
	v_vBrushTexCoord[1]	= a_vTexCoord0 * u_fBrushScale[1];
	v_vBrushTexCoord[2]	= a_vTexCoord0 * u_fBrushScale[2];
	v_vBrushTexCoord[3]	= a_vTexCoord0 * u_fBrushScale[3];

	highp vec4	vPosView = u_mModelView * a_vPosition;

	v_fFade		= 1.0 - clamp( (length( vPosView ) - SPLAT_FADE_START) / SPLAT_FADE_DISTANCE, 0.0, 1.0 );

	gl_Position	= u_mModelViewProj * a_vPosition;
}
#endif
#ifdef FSHADER
//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------
void main()
{
	lowp vec4	a	= texture2D( u_AlphaTexture, v_vAlphaTexCoord ) * v_fFade;

	lowp vec4	b0	= texture2D( u_BrushTexture[0], v_vBrushTexCoord[0] );
	lowp vec4	b1	= texture2D( u_BrushTexture[1], v_vBrushTexCoord[1] );
	lowp vec4	b2	= texture2D( u_BrushTexture[2], v_vBrushTexCoord[2] );
	lowp vec4	b3	= texture2D( u_BrushTexture[3], v_vBrushTexCoord[3] );

	const lowp vec4	c_vOne	= vec4( 1.0 );

	lowp vec4	vColor	= (b0 * a.r) + (b1 * a.g) + (b2 * a.b) + (b3 * a.a);
	
	gl_FragColor	= vec4( vColor.xyz, dot( a, c_vOne ) );
}
#endif
