// terrain_splat_shade.fx
//
//// global.fx

#ifdef HAS_OPENGL_ES
#	ifdef VSHADER
		precision highp float;
		precision highp int;
#	endif
#	ifdef FSHADER
		precision mediump float;
		precision mediump int;
#	endif
#else
#	define highp
#	define lowp 
#	define mediump
#endif //HAS_OPENGL_ES


//--------------------------------------------------------------------------------------------------
// Uniforms
//--------------------------------------------------------------------------------------------------
uniform highp mat4		u_mModelViewProj;
uniform highp mat4		u_mModelView;

uniform lowp sampler2D	u_LightmapTexture;
uniform lowp sampler2D	u_DetailmapTexture;

uniform mediump float	u_fDetailScale;

//--------------------------------------------------------------------------------------------------
// Varying
//--------------------------------------------------------------------------------------------------
varying mediump vec2	v_vTexCoord0;
varying mediump vec2	v_vDetailTexCoord;

//// packed_float.fx

const highp vec4	c_vPackFloatRGBAFactors		= vec4( 1.0, 255.0, 65025.0, 160581375.0 );
const highp vec4	c_vUnpackFloatRGBAFActors	= vec4( 1.0, 1.0/255.0, 1.0/65025.0, 1.0/160581375.0 );
//==================================================================================================
//==================================================================================================
highp vec4 PackFloatRGBA( float v )
{
  return fract( c_vPackFloatRGBAFactors * v );
}
//==================================================================================================
//==================================================================================================
float UnpackFloatRGBA( highp vec4 rgba ) 
{
  return dot( rgba, c_vUnpackFloatRGBAFActors );
}

//// shadowmap.fx

#ifdef SHADOWMAP

#ifdef SHADOWMAP_SAMPLERS
#	define ShadowSampler2D		sampler2DShadow
#	ifdef HAS_OPENGL_ES
#		extension GL_EXT_shadow_samplers : require
#	endif
#else
#	define ShadowSampler2D		sampler2D
#endif
//--------------------------------------------------------------------------------------------------
// Uniforms
//--------------------------------------------------------------------------------------------------
uniform highp mat4				u_mShadowProj[SHADOWMAP_NUM_SPLITS];
uniform highp ShadowSampler2D	u_sShadowTexture[SHADOWMAP_NUM_SPLITS];
uniform mediump vec3			u_vShadowViewSplits;					// The distance for each level in view space
//--------------------------------------------------------------------------------------------------
// Varying
//--------------------------------------------------------------------------------------------------
varying highp vec4				v_vPosView;

#ifdef VSHADER
//==================================================================================================
//==================================================================================================
void MakeShadowMapViewPos( mediump vec4 vPosView )
{
	v_vPosView		= vPosView;
}
#endif
#ifdef FSHADER
//==================================================================================================
//==================================================================================================
lowp float ShadowSample( ShadowSampler2D sShadowMap, highp vec4 vPosShadow )
{
#ifdef SHADOWMAP_SAMPLERS
	//----------------------------------------------------------------------------------------------
	//  When the hardware supports sampler2DShadow, the value returned from the sampler is a full
	// coverage value.  The sampling uses the z value of the texture coords to do the compare
	// after the sampling.  Also the hardware will do a 4x4 PCF if the texture filtering is set to
	// LINEAR.
	//----------------------------------------------------------------------------------------------
#	ifdef HAS_OPENGL_ES
		return shadow2DProjEXT( sShadowMap, vPosShadow );
#	else
		return shadow2DProj( sShadowMap, vPosShadow ).x;
#	endif
#else
	//----------------------------------------------------------------------------------------------
	//  The value retuned from a regular sampler2D is the z value for the light.  If the texture is 
	// a RGBA, we have to unpack into a float first.  Finally this z value is compare with the 
	// light z to give us a shade or not shaded answer.
	//----------------------------------------------------------------------------------------------
#ifdef SHADOWMAP_PACKED_FLOATRGBA
	highp float	fLightDepth	= UnpackFloatRGBA( texture2D( sShadowMap, vPosShadow.xy ) );
#else
	highp float	fLightDepth	= texture2D( sShadowMap, vPosShadow.xy ).z;
#endif

	return (vPosShadow.z <= fLightDepth) ? 1.0 : 0.0;
#endif	//SHADOWMAP_SAMPLERS
}
//==================================================================================================
//==================================================================================================
lowp vec4 ShadowMapMixAmbient( lowp vec4 vColor, lowp float fShade )
{
	return vColor * min( fShade + SHADOWMAP_AMBIENT, 1.0 );
}
//==================================================================================================
//==================================================================================================
lowp vec4 ShadowMapMixAmbientFade( lowp vec4 vColor, lowp float fShade )
{
	// Ramp the ambient for the shadows from SHADOWMAP_AMBIENT to 1.0 to make shadows fade.
	lowp float	fFade			= clamp( (length( v_vPosView ) - SHADOWMAP_FADE_START) / SHADOWMAP_FADE_DISTANCE, 0.0, 1.0 );
	lowp float	fShadowAmbient	= SHADOWMAP_AMBIENT + (SHADOWMAP_ONE_MIN_AMBIENT * fFade);

	return vColor * min( fShade + fShadowAmbient, 1.0 );
}
//==================================================================================================
//==================================================================================================
lowp vec4 ShadowMapShade( lowp vec4 vColor, highp vec4 vPosView )
{
	//----------------------------------------------------------------------------------------------
	//  Use the depth coord to determine which ShadowMap level we will use.  Each level has a 
	// projection and a texture.
	//----------------------------------------------------------------------------------------------
#if SHADOWMAP_NUM_SPLITS == 1

	lowp float	fShade = ShadowSample( u_sShadowTexture[0], u_mShadowProj[0] * vPosView );

	return ShadowMapMixAmbientFade( vColor, fShade );

#elif SHADOWMAP_NUM_SPLITS == 2

	if ( gl_FragCoord.z < u_vShadowViewSplits.x )
	{
		lowp float	fShade0 = ShadowSample( u_sShadowTexture[0], u_mShadowProj[0] * vPosView );

		return ShadowMapMixAmbient( vColor, fShade0 );
	}
	else
	{
		lowp float	fShade1 = ShadowSample( u_sShadowTexture[1], u_mShadowProj[1] * vPosView );

		return ShadowMapMixAmbientFade( vColor, fShade1 );
	}
#elif SHADOWMAP_NUM_SPLITS >= 3 

	if ( gl_FragCoord.z < u_vShadowViewSplits.x )
	{
		lowp float	fShade0 = ShadowSample( u_sShadowTexture[0], u_mShadowProj[0] * vPosView );

		return ShadowMapMixAmbient( vColor, fShade0 );
	}
	else
	if ( gl_FragCoord.z < u_vShadowViewSplits.y )
	{
		lowp float	fShade1 = ShadowSample( u_sShadowTexture[1], u_mShadowProj[1] * vPosView );

		return ShadowMapMixAmbient( vColor, fShade1 );
	}

	lowp float	fShade2 = ShadowSample( u_sShadowTexture[2], u_mShadowProj[2] * vPosView );

	return ShadowMapMixAmbientFade( vColor, fShade2 );
#endif
}
#endif	//FSHADER
#endif	//SHADOWMAP

#ifdef VSHADER
//--------------------------------------------------------------------------------------------------
// Vertex attributes
//--------------------------------------------------------------------------------------------------
attribute highp vec4	a_vPosition;
attribute mediump vec2	a_vTexCoord0;
//==================================================================================================
//==================================================================================================
void main()
{
	v_vTexCoord0		= a_vTexCoord0;
	v_vDetailTexCoord	= a_vTexCoord0 * u_fDetailScale;

#ifdef SHADOWMAP
	MakeShadowMapViewPos( u_mModelView * a_vPosition );
#endif
	gl_Position			= u_mModelViewProj * a_vPosition;
}
#endif
#ifdef FSHADER
//==================================================================================================
//==================================================================================================
void main()
{
	lowp vec4	vColor	= texture2D( u_LightmapTexture, v_vTexCoord0 ) * texture2D( u_DetailmapTexture, v_vDetailTexCoord );

#ifdef SHADOWMAP
	vColor	= ShadowMapShade( vColor, v_vPosView );
#endif

	gl_FragColor	= vColor;
}
#endif
