// world_meshes.fx
//
//// global.fx

#ifdef HAS_OPENGL_ES
#	ifdef VSHADER
		precision highp float;
		precision highp int;
#	endif
#	ifdef FSHADER
		precision mediump float;
		precision mediump int;
#	endif
#else
#	define highp
#	define lowp 
#	define mediump
#endif //HAS_OPENGL_ES

//

//--------------------------------------------------------------------------------------------------
// Uniforms
//--------------------------------------------------------------------------------------------------
uniform mat4		u_mModelViewProj;
uniform mat4		u_mModelView;

uniform sampler2D	u_Texture;

//--------------------------------------------------------------------------------------------------
// Varying
//--------------------------------------------------------------------------------------------------
varying vec2		v_vTexCoord0;

//// fog.fx
//

#ifdef FOG

//--------------------------------------------------------------------------------------------------
// Uniforms
//--------------------------------------------------------------------------------------------------
uniform mediump float	u_fFogEnd;
uniform mediump float	u_fFogDensity;			// end-start for linear
uniform lowp vec3		u_vFogColor;

//--------------------------------------------------------------------------------------------------
// Varying
//--------------------------------------------------------------------------------------------------
varying lowp float		v_fFogShade;

#ifdef VSHADER
//==================================================================================================
//==================================================================================================
void MakeFogShade( mediump float fEyeDistance )
{
	v_fFogShade = clamp( (u_fFogEnd - fEyeDistance) / u_fFogDensity, 0.0, 1.0 );
}
#endif
#ifdef FSHADER
//==================================================================================================
//==================================================================================================
lowp vec4 FogShade( lowp vec4 vColor )
{
    return vec4( mix( u_vFogColor, vec3( vColor ), v_fFogShade ), vColor.a );
}
#endif

#endif	//FOG


#ifdef VSHADER
//--------------------------------------------------------------------------------------------------
// Vertex attributes
//--------------------------------------------------------------------------------------------------
attribute vec4		a_vPosition;
attribute vec2		a_vTexCoord;
//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------
void main()
{
	v_vTexCoord0	= a_vTexCoord;

#ifdef FOG
	// Linear per vertex FOG.
	mediump vec4	vPosView	= u_mModelView * a_vPosition;

	MakeFogShade( abs( vPosView.z ) );
#endif

	gl_Position		= u_mModelViewProj * a_vPosition;
}
#endif
#ifdef FSHADER
//--------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------
void main()
{
	lowp vec4	vColor	= texture2D( u_Texture, v_vTexCoord0 );

#ifdef FOG
	// Mix in fog.
    vColor	= FogShade( vColor );
#endif

	gl_FragColor	= vColor;
}
#endif
